import os
import itertools

from django.conf import settings
from django.contrib.staticfiles import finders


def static_files_iterator(tested_locations, extension, ignore_patterns=None, additional_settings_list=None):
    if ignore_patterns is None:
        ignore_patterns = []

    source = (os.path.join(storage.location, path)
              for finder in finders.get_finders()
              for path, storage in finder.list(ignore_patterns))

    if additional_settings_list and hasattr(settings, additional_settings_list):
        source = itertools.chain(source, getattr(settings, additional_settings_list))

    return (path for path in source
            if path.endswith(extension)
            if any(path.startswith(location) for location in tested_locations))



import os
import sys
import warnings
from optparse import OptionParser, make_option

from django.conf import settings
from django.core.management.commands.test import Command as TestCommand
from django.utils.importlib import import_module

from django_jenkins.runner import CITestSuiteRunner


def get_runner(settings, test_runner_class=None):
    if test_runner_class is None:
        test_runner_class = getattr(
            settings, 'JENKINS_TEST_RUNNER', 'django_jenkins.runner.CITestSuiteRunner')

    test_module_name, test_cls = test_runner_class.rsplit('.', 1)
    test_module = __import__(test_module_name, {}, {}, test_cls)
    test_runner = getattr(test_module, test_cls)

    if not issubclass(test_runner, CITestSuiteRunner):
        raise ValueError('Your custom TestRunner should extend '
                         'the CITestSuiteRunner class.')
    return test_runner


class Command(TestCommand):
    option_list = TestCommand.option_list + (
        make_option('--output-dir', dest='output_dir', default="reports",
                    help='Report files directory'),
        make_option("--enable-coverage",
                    action="store_true", default=False,
                    help="Measure code coverage"),
        make_option('--debug', action='store_true',
                    dest='debug', default=False,
                    help='Do not intercept stdout and stderr, friendly for console debuggers'),
        make_option("--coverage-rcfile",
                    dest="coverage_rcfile",
                    default="",
                    help="Specify configuration file."),
        make_option("--coverage-html-report",
                    dest="coverage_html_report_dir",
                    default="",
                    help="Directory to which HTML coverage report should be"
                    " written. If not specified, no report is generated."),
        make_option("--coverage-exclude", action="append",
                    default=[], dest="coverage_excludes",
                    help="Module name to exclude")
    )

    def __init__(self):
        super(Command, self).__init__()
        self.tasks_cls = [import_module(module_name).Reporter
                          for module_name in self.get_task_list()]
        self.tasks = [task_cls() for task_cls in self.tasks_cls]

    def get_task_list(self):
        tasks = getattr(settings, 'JENKINS_TASKS', ())
        if '--enable-coverage' in sys.argv and 'django_jenkins.tasks.with_coverage' not in tasks:
            try:
                from django.apps import apps  # NOQA
            except ImportError:
                """
                We are on django 1.6
                """
                tasks += ('django_jenkins.tasks.with_coverage',)
        return tasks

    def create_parser(self, prog_name, subcommand):
        test_runner_class = get_runner(settings, self.test_runner)
        options = self.option_list + getattr(
            test_runner_class, 'option_list', ())

        for task in self.tasks:
            options += tuple(option for option in getattr(task, 'option_list', ())
                             if all(option._long_opts[0] != existing._long_opts[0]
                                    for existing in options))

        return OptionParser(prog=prog_name,
                            usage=self.usage(subcommand),
                            version=self.get_version(),
                            option_list=options)

    def handle(self, *test_labels, **options):
        TestRunner = get_runner(settings, options.get('testrunner'))
        options['verbosity'] = int(options.get('verbosity'))

        if options.get('liveserver') is not None:
            os.environ['DJANGO_LIVE_TEST_SERVER_ADDRESS'] = options['liveserver']
            del options['liveserver']

        output_dir = options['output_dir']
        if not os.path.exists(output_dir):
            os.makedirs(output_dir)

        test_runner = TestRunner(**options)
        failures = test_runner.run_tests(test_labels)

        if failures:
            sys.exit(bool(failures))
        else:
            tested_locations = self.get_tested_locations(test_labels)

            # dump coverage
            try:
                from django.apps import apps
                coverage = apps.get_app_config('django_jenkins').coverage
                if coverage:
                    if options['verbosity'] >= 1:
                        print('Storing coverage info...')

                    coverage.save(tested_locations, options)
            except ImportError:
                """
                Do nothing on django 1.6
                """

            # run reporters
            for task in self.tasks:
                if options['verbosity'] >= 1:
                    print('Executing {0}...'.format(task.__module__))
                task.run(tested_locations, **options)

            if options['verbosity'] >= 1:
                print('Done')

    def get_tested_locations(self, test_labels):
        locations = []

        if test_labels:
            pass
        elif hasattr(settings, 'PROJECT_APPS'):
            test_labels = settings.PROJECT_APPS
        else:
            warnings.warn('No PROJECTS_APPS settings, coverage gathered over all apps')
            test_labels = settings.INSTALLED_APPS

        try:
            from django.apps import apps
            for test_label in test_labels:
                app_config = apps.get_containing_app_config(test_label)
                locations.append(os.path.dirname(app_config.module.__file__))
        except ImportError:
            # django 1.6
            from django.db.models import get_app
            for test_label in test_labels:
                models_module = get_app(test_label.split('.')[-1])
                locations.append(os.path.dirname(models_module.__file__))

        return locations



import os
import sys
import time

from xml.etree import ElementTree as ET

from django.conf import settings
from django.test.runner import DiscoverRunner
from django.utils.encoding import smart_text
from django.utils.unittest import TextTestResult, TextTestRunner


class EXMLTestResult(TextTestResult):
    def __init__(self, *args, **kwargs):
        self.case_start_time = time.time()
        self.run_start_time = None
        self.tree = None
        super(EXMLTestResult, self).__init__(*args, **kwargs)

    def startTest(self, test):
        self.case_start_time = time.time()
        super(EXMLTestResult, self).startTest(test)

    def startTestRun(self):
        self.tree = ET.Element('testsuite')
        self.run_start_time = time.time()
        super(EXMLTestResult, self).startTestRun()

    def addSuccess(self, test):
        self.testcase = self._make_testcase_element(test)
        super(EXMLTestResult, self).addSuccess(test)

    def addFailure(self, test, err):
        self.testcase = self._make_testcase_element(test)
        test_result = ET.SubElement(self.testcase, 'failure')
        self._add_tb_to_test(test, test_result, err)
        super(EXMLTestResult, self).addFailure(test, err)

    def addError(self, test, err):
        self.testcase = self._make_testcase_element(test)
        test_result = ET.SubElement(self.testcase, 'error')
        self._add_tb_to_test(test, test_result, err)
        super(EXMLTestResult, self).addError(test, err)

    def addUnexpectedSuccess(self, test):
        self.testcase = self._make_testcase_element(test)
        test_result = ET.SubElement(self.testcase, 'skipped')
        test_result.set('message', 'Test Skipped: Unexpected Success')
        super(EXMLTestResult, self).addUnexpectedSuccess(test)

    def addSkip(self, test, reason):
        self.testcase = self._make_testcase_element(test)
        test_result = ET.SubElement(self.testcase, 'skipped')
        test_result.set('message', 'Test Skipped: %s' % reason)
        super(EXMLTestResult, self).addSkip(test, reason)

    def addExpectedFailure(self, test, err):
        self.testcase = self._make_testcase_element(test)
        test_result = ET.SubElement(self.testcase, 'skipped')
        self._add_tb_to_test(test, test_result, err)
        super(EXMLTestResult, self).addExpectedFailure(test, err)

    def stopTest(self, test):
        if self.buffer:
            output = sys.stdout.getvalue() if hasattr(sys.stdout, 'getvalue') else ''
            if output:
                sysout = ET.SubElement(self.testcase, 'system-out')
                sysout.text = smart_text(output, errors='ignore')

            error = sys.stderr.getvalue() if hasattr(sys.stderr, 'getvalue') else ''
            if error:
                syserr = ET.SubElement(self.testcase, 'system-err')
                syserr.text = smart_text(error, errors='ignore')

        super(EXMLTestResult, self).stopTest(test)

    def stopTestRun(self):
        run_time_taken = time.time() - self.run_start_time
        self.tree.set('name', 'Django Project Tests')
        self.tree.set('errors', str(len(self.errors)))
        self.tree.set('failures', str(len(self.failures)))
        self.tree.set('skips', str(len(self.skipped)))
        self.tree.set('tests', str(self.testsRun))
        self.tree.set('time', "%.3f" % run_time_taken)
        super(EXMLTestResult, self).stopTestRun()

    def _make_testcase_element(self, test):
        time_taken = time.time() - self.case_start_time
        classname = ('%s.%s' % (test.__module__, test.__class__.__name__)).split('.')
        testcase = ET.SubElement(self.tree, 'testcase')
        testcase.set('time', "%.6f" % time_taken)
        testcase.set('classname', '.'.join(classname))
        testcase.set('name', getattr(test, '_testMethodName',
                                     getattr(test, 'description', 'UNKNOWN')))
        return testcase

    def _add_tb_to_test(self, test, test_result, err):
        '''Add a traceback to the test result element'''
        exc_class, exc_value, tb = err
        tb_str = self._exc_info_to_string(err, test)
        test_result.set('type', '%s.%s' % (exc_class.__module__, exc_class.__name__))
        test_result.set('message', smart_text(exc_value))
        test_result.text = tb_str

    def dump_xml(self, output_dir):
        """
        Dumps test result to xml
        """
        output = ET.ElementTree(self.tree)
        output.write(os.path.join(output_dir, 'junit.xml'), encoding="utf-8")


class CITestSuiteRunner(DiscoverRunner):
    def __init__(self, output_dir='reports', debug=False, **kwargs):
        self.output_dir = output_dir
        self.debug = debug
        super(CITestSuiteRunner, self).__init__(**kwargs)

        if not hasattr(self, 'test_runner'):
            # django 1.6 compatibility
            self.test_runner = TextTestRunner

    def setup_databases(self):
        if 'south' in settings.INSTALLED_APPS:
            from south.management.commands import patch_for_test_db_setup
            patch_for_test_db_setup()
        return super(CITestSuiteRunner, self).setup_databases()

    def run_suite(self, suite, **kwargs):
        result = self.test_runner(
            verbosity=self.verbosity,
            failfast=self.failfast,
            resultclass=EXMLTestResult,
            buffer=not self.debug
        ).run(suite)

        result.dump_xml(self.output_dir)

        return result



# -*- coding: utf-8 -*-
from django.conf.urls import patterns, url
from django.views.generic.base import TemplateView

urlpatterns = patterns('',  # NOQA
     url(r'^test_click/$', TemplateView.as_view(template_name='test_app/wm_test_click.html'), name='wm_test_click')
)



# -*- coding: utf-8 -*-
import os
import re
import sys
from pyflakes.scripts import pyflakes

try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO


class Reporter(object):
    def run(self, apps_locations, **options):
        output = open(os.path.join(options['output_dir'], 'pyflakes.report'), 'w')

        # run pyflakes tool with captured output
        old_stdout, pyflakes_output = sys.stdout, StringIO()
        sys.stdout = pyflakes_output
        try:
            for location in apps_locations:
                if os.path.isdir(location):
                    for dirpath, dirnames, filenames in os.walk(os.path.relpath(location)):
                        if dirpath.endswith(os.sep + 'south_migrations'):
                            continue

                        for filename in filenames:
                            if filename.endswith('.py'):
                                pyflakes.checkPath(os.path.join(dirpath, filename))
                else:
                    pyflakes.checkPath(os.path.relpath(location))
        finally:
            sys.stdout = old_stdout

        # save report
        pyflakes_output.seek(0)

        while True:
            line = pyflakes_output.readline()
            if not line:
                break
            message = re.sub(r': ', r': [E] PYFLAKES:', line)
            output.write(message)

        output.close()



import sys
from django.apps import AppConfig
from django_jenkins.tasks.with_coverage import CoverageReporter


class JenkinsConfig(AppConfig):
    """
    Enable coverage measurement as soon as possible
    """
    name = 'django_jenkins'

    def __init__(self, app_name, app_module):
        super(JenkinsConfig, self).__init__(app_name, app_module)

        self.coverage = None

        if 'jenkins' in sys.argv and '--enable-coverage' in sys.argv:
            """
            Starting coverage as soon as possible
            """
            self.coverage = CoverageReporter()



# -*- coding: utf-8 -*-
from lettuce import step, before

@before.each_scenario
def setup_scenario(scenario):
    scenario.numbers = []

@step('(?:Given|And|When) the number "(.*)"(?: is added to (?:it|them))?')
def given_the_number(step, number):
    step.scenario.numbers.append(int(number))

@step('Then the result should be "(.*)"')
def then_the_result_should_equal(step, result):
    actual = sum(step.scenario.numbers)
    assert int(result) == actual, "%s != %s" % (result, actual)



from south.v2 import SchemaMigration

class Migration(SchemaMigration):
    def forwards(self, orm):
        a = 1 # pyflakes/pylint violation
        pass

    def backwards(self, orm):
        pass



import warnings
import os
import sys
from django.conf import settings
from django.utils.importlib import import_module


def default_coverage_config():
    rcfile = getattr(settings, 'COVERAGE_RCFILE', 'coverage.rc')
    if os.path.exists(rcfile):
        return rcfile
    return None


class CoverageReporter(object):
    def __init__(self):
        try:
            from coverage.control import coverage
        except ImportError:
            raise ImportError('coverage is not installed')
        else:
            coverage_config_file = None
            for argv in sys.argv:
                if argv.startswith('--coverage-rcfile='):
                    _, coverage_config_file = argv.split('=')

            self.coverage = coverage(
                branch=True,
                config_file=coverage_config_file or default_coverage_config())
            self.coverage.start()

    def save(self, apps_locations, options):
        self.coverage.stop()
        self.coverage._harvest_data()
        morfs = self.get_morfs(self.coverage, apps_locations, options)

        self.coverage.xml_report(morfs=morfs, outfile=os.path.join(options['output_dir'], 'coverage.xml'))

        # Dump coverage html
        coverage_html_dir = options.get('coverage_html_report_dir') \
            or getattr(settings, 'COVERAGE_REPORT_HTML_OUTPUT_DIR', '')
        if coverage_html_dir:
            self.coverage.html_report(morfs=morfs, directory=coverage_html_dir)

    def get_morfs(self, coverage, tested_locations, options):
        excluded = []

        # Exclude by module
        modnames = options.get('coverage_excludes') or getattr(settings, 'COVERAGE_EXCLUDES', [])
        for modname in modnames:
            try:
                excluded.append(os.path.dirname(import_module(modname).__file__))
            except ImportError:
                pass

        # Exclude by directory
        excluded.extend(getattr(settings, 'COVERAGE_EXCLUDES_FOLDERS', []))

        return [filename for filename in coverage.data.measured_files()
                if not (os.sep + 'migrations' + os.sep) in filename
                if not (os.sep + 'south_migrations' + os.sep) in filename
                if any(filename.startswith(location) for location in tested_locations)
                if not any(filename.startswith(location) for location in excluded)]


class Reporter(object):
    def __init__(self):
        try:
            from django.apps import apps
            self.coverage_reporter = apps.get_app_config('django_jenkins').coverage
            if self.coverage_reporter is None:
                warnings.warn('django_jenkins.tasks.with_coverage is depricated.'
                              ' Please, delete it from JENKINS_TASKS and use --enable-coverage'
                              ' command line option instead')
                self.coverage_reporter = CoverageReporter()
        except ImportError:
            # Ok, on django 1.6 work as usual
            self.coverage_reporter = CoverageReporter()

    def run(self, apps_locations, **options):
        self.coverage_reporter.save(apps_locations, options)



# -*- coding: utf-8 -*-
import os
import codecs
import subprocess
from optparse import make_option
from django_jenkins.tasks import static_files_iterator


class Reporter(object):
    option_list = (
        make_option("--jshint-exclude",
                    dest="jshint_exclude", default="",
                    help="Exclude patterns"),
    )

    def run(self, apps_locations, **options):
        output = codecs.open(os.path.join(options['output_dir'], 'jshint.xml'), 'w', 'utf-8')

        files = list(
            static_files_iterator(apps_locations,
                                  extension='.js',
                                  ignore_patterns=options['jshint_exclude'].split(',')))

        cmd = ['jshint', '--jslint-reporter'] + files

        process = subprocess.Popen(cmd, stdout=subprocess.PIPE)
        jshint_output, err = process.communicate()
        retcode = process.poll()
        if retcode not in [0, 1, 2]:  # normal jshint return codes
            raise subprocess.CalledProcessError(retcode, cmd, output='%s\n%s' % (jshint_output, err))

        output.write(jshint_output.decode('utf-8'))
        output.close()



import os

PROJECT_ROOT = os.path.abspath(os.path.dirname(__file__))

DEBUG = True
TEMPLATE_DEBUG = DEBUG
ROOT_URLCONF = 'test_app.urls'
SECRET_KEY = 'nokey'
MIDDLEWARE_CLASSES = ()

TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.Loader',
    'django.template.loaders.app_directories.Loader',
)

PROJECT_APPS = (
    'django.contrib.sessions',  # just to ensure that dotted apps test works
    'django_jenkins',
    'test_app',
    'test_app_dirs',
)

INSTALLED_APPS = (
    'django.contrib.contenttypes',
) + PROJECT_APPS


DATABASE_ENGINE = 'sqlite3'
DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.%s' % DATABASE_ENGINE,
        }
}

SOUTH_MIGRATION_MODULES = {
    'test_app': 'test_app.south_migrations',
}

JENKINS_TASKS = (
    'django_jenkins.tasks.run_pylint',
    'django_jenkins.tasks.run_pep8',
    'django_jenkins.tasks.run_csslint',
    'django_jenkins.tasks.run_flake8',
    'django_jenkins.tasks.run_pyflakes',
    'django_jenkins.tasks.run_jshint',
    'django_jenkins.tasks.run_sloccount',
)


JSHINT_CHECKED_FILES = [os.path.join(PROJECT_ROOT, 'static/js/test.js')]
CSSLINT_CHECKED_FILES = [os.path.join(PROJECT_ROOT, 'static/css/test.css')]


STATIC_URL = '/media/'


LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'handlers': {
        'console': {
            'level': 'DEBUG',
            'class': 'logging.StreamHandler',
        },
    },
    'loggers': {
        'django.request': {
            'handlers': ['console'],
            'level': 'ERROR',
            'propagate': True,
        },
    }
}



# -*- coding: utf-8; mode: django -*-
import os
import subprocess
import codecs
from optparse import make_option
from django_jenkins.tasks import static_files_iterator


class Reporter(object):
    option_list = (
        make_option("--csslint-exclude",
                    dest="csslint_exclude", default=".min.css",
                    help="Comma separated exclude file patterns"),
        make_option("--csslint-ignore",
                    dest="csslint_ignore", default="",
                    help="CSSLint Ignore rules")
    )

    def run(self, apps_locations, **options):
        output = codecs.open(os.path.join(options['output_dir'], 'csslint.report'), 'w', 'utf-8')

        files = list(
            static_files_iterator(apps_locations,
                                  extension='.css',
                                  ignore_patterns=options['csslint_exclude'].split(',')))

        if files:
            cmd = ['csslint', '--format=lint-xml'] + files

            if options['csslint_ignore']:
                cmd += ['--ignore=%s' % options['csslint_ignore']]

            process = subprocess.Popen(cmd, stdout=subprocess.PIPE)
            process_output, err = process.communicate()
            retcode = process.poll()
            if retcode not in [0, 1]:  # normal csslint return codes
                raise subprocess.CalledProcessError(retcode, cmd, output=output + '\n' + err)

            output.write(process_output.decode('utf-8'))
        else:
            output.write('<?xml version="1.0" encoding='
                         '"utf-8"?><lint></lint>')

        output.close()



# -*- coding: utf-8 -*-
import sys
from django.core import mail
from django.test import TestCase
from django.utils.unittest import skip
from django.test import LiveServerTestCase
from selenium.webdriver.firefox.webdriver import WebDriver


class SaintyChecks(TestCase):
    #@classmethod
    #def setUpClass(cls):
    #    raise Exception("Ups, should be disabled")

    def test_mailbox_stubs_not_broken(self):
        print("Testing mailbox django stubs")
        mail.send_mail('Test subject', 'Test message', 'nobody@kenkins.com',
                       ['somewhere@nowhere.com'])
        self.assertTrue(1, len(mail.outbox))

    @skip("Check skiped test")
    def test_is_skipped(self):
        print("This test should be skipped")

    def test_junit_xml_with_utf8_stdout_and_stderr(self):
        sys.stdout.write('\xc4\x85')
        sys.stderr.write('\xc4\x85')

    def test_junit_xml_with_invalid_stdout_and_stderr_encoding(self):
        sys.stdout.write('\xc4')
        sys.stderr.write('\xc4')


    #def test_failure(self):
    #    raise Exception("Ups, should be disabled")


class SeleniumTests(LiveServerTestCase):
    fixtures = ['default_users.json']

    @classmethod
    def setUpClass(cls):
        cls.selenium = WebDriver()
        super(SeleniumTests, cls).setUpClass()

    @classmethod
    def tearDownClass(cls):
        super(SeleniumTests, cls).tearDownClass()
        cls.selenium.quit()

    def test_login(self):
        self.selenium.get('%s%s' % (self.live_server_url, '/test_click/'))
        self.selenium.find_element_by_id("wm_click").click()
        self.assertEqual('Button clicked', self.selenium.find_element_by_id("wm_target").text)



# -*- coding: utf-8 -*-
import os
import subprocess


class Reporter(object):
    def run(self, apps_locations, **options):
        output = open(os.path.join(options['output_dir'], 'sloccount.report'), 'w')
        cmd = ['sloccount', "--duplicates", "--wide", "--details"] + apps_locations

        process = subprocess.Popen(cmd, stdout=subprocess.PIPE)
        report_output, err = process.communicate()

        retcode = process.poll()
        if retcode not in [0]:  # normal sloccount return codes
            raise subprocess.CalledProcessError(retcode, cmd, output='%s\n%s' % (report_output, err))

        output.write(report_output.decode('utf-8'))
        output.close()



import os
from optparse import make_option
from django.conf import settings

from pylint import lint
from pylint.reporters.text import TextReporter


class ParseableTextReporter(TextReporter):
    """
    Outputs messages in a form recognized by jenkins

    <filename>:<linenum>:<msg>
    """
    name = 'parseable'
    line_format = '{path}:{line}: [{msg_id}({symbol}), {obj}] {msg}'


class Reporter(object):
    option_list = (
        make_option("--pylint-rcfile",
                    dest="pylint_rcfile",
                    help="pylint configuration file"),
        make_option("--pylint-errors-only",
                    dest="pylint_errors_only",
                    action="store_true", default=False,
                    help="pylint output errors only mode")
    )

    def run(self, apps_locations, **options):
        output = open(os.path.join(options['output_dir'], 'pylint.report'), 'w')

        args = ["--rcfile=%s" % self.get_config_path(options)]
        if options['pylint_errors_only']:
            args += ['--errors-only']
        args += apps_locations

        lint.Run(args, reporter=ParseableTextReporter(output=output), exit=False)

    def get_config_path(self, options):
        if options['pylint_rcfile']:
            return options['pylint_rcfile']

        rcfile = getattr(settings, 'PYLINT_RCFILE', 'pylint.rc')
        if os.path.exists(rcfile):
            return rcfile

        # use build-in
        root_dir = os.path.normpath(os.path.dirname(__file__))
        return os.path.join(root_dir, 'pylint.rc')



# -*- coding: utf-8 -*-
import os, sys
from django.core.management import execute_from_command_line

PROJECT_ROOT = os.path.dirname(os.path.abspath(os.path.dirname(__file__)))
sys.path.insert(0, PROJECT_ROOT)

if __name__ == "__main__":
    if len(sys.argv) == 1:
        sys.argv += ['test']

    os.environ.setdefault("DJANGO_SETTINGS_MODULE", "tests.settings")
    execute_from_command_line(sys.argv)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import codecs
from os import path
from setuptools import setup


read = lambda filepath: codecs.open(filepath, 'r', 'utf-8').read()


setup(
    name='django-jenkins',
    version='0.16.2',
    author='Mikhail Podgurskiy',
    author_email='kmmbvnr@gmail.com',
    description='Plug and play continuous integration with django and jenkins',
    long_description=read(path.abspath(path.join(path.dirname(__file__), 'README.rst'))),
    license='LGPL',
    platforms=['Any'],
    keywords=['pyunit', 'unittest', 'testrunner', 'hudson', 'jenkins',
              'django', 'pylint', 'pep8', 'pyflakes', 'csslint', 'jshint',
              'coverage'],
    url='http://github.com/kmmbvnr/django-jenkins',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Software Development :: Testing'
    ],
    install_requires=[
        'Django>=1.6',
    ],
    packages=['django_jenkins', 'django_jenkins.management',
              'django_jenkins.tasks', 'django_jenkins.management.commands'],
    package_data={'django_jenkins': ['tasks/pylint.rc']},
    zip_safe=False,
    include_package_data=True
)



import os
import pep8

from flake8.engine import get_style_guide
from django.conf import settings
from optparse import make_option


class Reporter(object):
    """
    Runs flake8 on python files.
    """
    option_list = (
        make_option('--max-complexity',
                    dest='max_complexity',
                    default='-1',
                    help='McCabe complexity treshold'),
        make_option("--pep8-exclude",
                    dest="pep8-exclude",
                    default=pep8.DEFAULT_EXCLUDE + ",south_migrations",
                    help="exclude files or directories which match these "
                    "comma separated patterns (default: %s)" %
                    pep8.DEFAULT_EXCLUDE),
        make_option("--pep8-select", dest="pep8-select",
                    help="select errors and warnings (e.g. E,W6)"),
        make_option("--pep8-ignore", dest="pep8-ignore",
                    help="skip errors and warnings (e.g. E4,W)"),
        make_option("--pep8-max-line-length",
                    dest="pep8-max-line-length", type='int',
                    help="set maximum allowed line length (default: %d)" %
                    pep8.MAX_LINE_LENGTH),
        make_option("--pep8-rcfile", dest="pep8-rcfile",
                    help="PEP8 configuration file"),
    )

    def run(self, apps_locations, **options):
        output = open(os.path.join(options['output_dir'], 'flake8.report'), 'w')

        class JenkinsReport(pep8.BaseReport):
            def error(instance, line_number, offset, text, check):
                code = super(JenkinsReport, instance).error(line_number, offset, text, check)

                if not code:
                    return
                sourceline = instance.line_offset + line_number
                output.write('%s:%s:%s: %s\n' % (instance.filename, sourceline, offset + 1, text))

        pep8_options = {'exclude': options['pep8-exclude'].split(',')}
        if options['pep8-select']:
            pep8_options['select'] = options['pep8-select'].split(',')
        if options['pep8-ignore']:
            pep8_options['ignore'] = options['pep8-ignore'].split(',')
        if options['pep8-max-line-length']:
            pep8_options['max_line_length'] = options['pep8-max-line-length']

        pep8style = get_style_guide(
            parse_argv=False,
            config_file=self.get_config_path(options),
            reporter=JenkinsReport,
            max_complexity=int(options['max_complexity']),
            jobs='1',
            **pep8_options)

        pep8style.options.report.start()
        for location in apps_locations:
            pep8style.input_dir(os.path.relpath(location))
        pep8style.options.report.stop()

        output.close()

    def get_config_path(self, options):
        if options['pep8-rcfile']:
            return options['pep8-rcfile']

        rcfile = getattr(settings, 'PEP8_RCFILE', None)
        if rcfile:
            return rcfile

        if os.path.exists('tox.ini'):
            return 'tox.ini'

        if os.path.exists('setup.cfg'):
            return 'setup.cfg'
